@@ -26,9 +26,16 @@
 
 #ifdef GCONF_ENABLE_INTERNALS
 
+#include <config.h>
+#include <libintl.h>
+
+#define _(String) dgettext (GETTEXT_PACKAGE, String)
+#define N_(String) (String)
+
 #include <glib.h>
 #include "gconf-error.h"
 #include "gconf-value.h"
+#include "gconf-engine.h"
 #include "GConf.h"
 
 gchar*       gconf_key_directory  (const gchar* key);
@@ -46,8 +53,8 @@ gboolean gconf_file_test   (const gchar* filename, int test);
 gboolean gconf_file_exists (const gchar* filename);
 
 GConfValue*  gconf_value_from_corba_value      (const ConfigValue *value);
-ConfigValue* corba_value_from_gconf_value      (GConfValue        *value);
-void         fill_corba_value_from_gconf_value (GConfValue        *value,
+ConfigValue* corba_value_from_gconf_value      (const GConfValue  *value);
+void         fill_corba_value_from_gconf_value (const GConfValue  *value,
                                                 ConfigValue       *dest);
 ConfigValue* invalid_corba_value               (void);
 
@@ -55,16 +62,16 @@ ConfigValue* invalid_corba_value               (void);
 gchar* gconf_object_to_string (CORBA_Object obj,
                                GError **err);
 
-void          fill_corba_schema_from_gconf_schema (GConfSchema        *sc,
+void          fill_corba_schema_from_gconf_schema (const GConfSchema  *sc,
                                                    ConfigSchema       *dest);
-ConfigSchema* corba_schema_from_gconf_schema      (GConfSchema        *sc);
+ConfigSchema* corba_schema_from_gconf_schema      (const GConfSchema  *sc);
 GConfSchema*  gconf_schema_from_corba_schema      (const ConfigSchema *cs);
 
 const gchar*   gconf_value_type_to_string   (GConfValueType  type);
 GConfValueType gconf_value_type_from_string (const gchar    *str);
 
 
-gchar**       gconf_load_source_path (const gchar* filename, GError** err);
+GSList*       gconf_load_source_path (const gchar* filename, GError** err);
 
 /* shouldn't be used in applications (although implemented in gconf.c) */
 
@@ -104,10 +111,6 @@ gboolean      gconf_key_check(const gchar* key, GError** err);
  * If these were public they'd be in gconf-value.h
  */
 
-/* doesn't work on complicated types (only string, int, bool, float) */
-GConfValue* gconf_value_new_from_string      (GConfValueType type,
-                                              const gchar* str,
-                                              GError** err);
 /* for the complicated types */
 GConfValue* gconf_value_new_list_from_string (GConfValueType list_type,
                                               const gchar* str,
@@ -200,34 +203,53 @@ ConfigServer gconf_activate_server (gboolean  start_if_not_found,
 char*     gconf_get_lock_dir (void);
 char*     gconf_get_daemon_dir (void);
 
-gboolean gconf_schema_validate (GConfSchema *sc,
-                                GError     **err);
-gboolean gconf_value_validate  (GConfValue *value,
-                                GError    **err);
+gboolean gconf_schema_validate (const GConfSchema  *sc,
+                                GError            **err);
+gboolean gconf_value_validate  (const GConfValue   *value,
+                                GError            **err);
 
-#define g_utf8_validate gconf_g_utf8_validate
 
-gboolean gconf_g_utf8_validate (const gchar  *str,
-                                gssize        max_len,    
-                                const gchar **end);
+void gconf_engine_set_owner        (GConfEngine *engine,
+                                    gpointer     client);
+void gconf_engine_push_owner_usage (GConfEngine *engine,
+                                    gpointer     client);
+void gconf_engine_pop_owner_usage  (GConfEngine *engine,
+                                    gpointer     client);
 
-#ifdef ENABLE_NLS
-#    include <libintl.h>
-#    include <config.h>
-#    undef _
-#    define _(String) dgettext (GETTEXT_PACKAGE, String)
-#    ifdef gettext_noop
-#        define N_(String) gettext_noop (String)
-#    else
-#        define N_(String) (String)
-#    endif
-#else
-/* Stubs that do something close enough.  */
-#    define textdomain(String) (String)
-#    define bindtextdomain(Domain,Directory) (Domain)
-#    define _(String) (String)
-#    define N_(String) (String)
-#endif
+
+static inline gboolean
+gconf_CORBA_Object_equal (gconstpointer a, gconstpointer b)
+{
+  CORBA_Environment ev;
+  CORBA_Object _obj_a = (gpointer)a;
+  CORBA_Object _obj_b = (gpointer)b;
+  gboolean retval;
+
+  CORBA_exception_init (&ev);
+  retval = CORBA_Object_is_equivalent(_obj_a, _obj_b, &ev);
+  CORBA_exception_free (&ev);
+
+  return retval;
+}
+
+static inline guint
+gconf_CORBA_Object_hash(gconstpointer key)
+{
+  CORBA_Environment ev;
+  CORBA_Object _obj = (gpointer)key;
+  CORBA_unsigned_long retval;
+
+  CORBA_exception_init (&ev);
+  retval = CORBA_Object_hash(_obj, G_MAXUINT, &ev);
+  CORBA_exception_free (&ev);
+
+  return retval;
+}
+
+/* FIXME move to public eventually */
+GConfEntry* gconf_entry_copy (const GConfEntry *src);
+int         gconf_value_compare (const GConfValue *value_a,
+                                 const GConfValue *value_b);
 
 #endif /* GCONF_ENABLE_INTERNALS */
 
