@@ -32,11 +32,10 @@
 #include "gconf-listeners.h"
 #include "gconf-locale.h"
 #include "gconf-schema.h"
-#include "gconf-glib-private.h"
 #include "gconf.h"
 #include "gconfd.h"
 #include "gconf-database.h"
-#include <orb/orbit.h>
+#include <orbit/orbit.h>
 
 #include "GConf.h"
 
@@ -52,7 +51,6 @@
 #include <ctype.h>
 #include <syslog.h>
 #include <time.h>
-#include <liboaf/liboaf.h>
 
 /* This makes hash table safer when debugging */
 #ifndef GCONF_ENABLE_DEBUG
@@ -259,7 +257,7 @@ gconfd_shutdown(PortableServer_Servant servant, CORBA_Environment *ev)
 static void
 gconf_server_load_sources(void)
 {
-  gchar** addresses;
+  GSList* addresses;
   GList* tmp;
   gboolean have_writable = FALSE;
   gchar* conffile;
@@ -289,13 +287,9 @@ gconf_server_load_sources(void)
   if (addresses == NULL)
     {      
       /* Try using the default address xml:readwrite:$(HOME)/.gconf */
-      addresses = g_new0(gchar*, 2);
+      addresses = g_slist_append(addresses, g_strconcat("xml:readwrite:", g_get_home_dir(), "/.gconf", NULL));
 
-      addresses[0] = g_strconcat("xml:readwrite:", g_get_home_dir(), "/.gconf", NULL);
-
-      addresses[1] = NULL;
-      
-      gconf_log(GCL_DEBUG, _("No configuration files found, trying to use the default config source `%s'"), addresses[0]);
+      gconf_log(GCL_DEBUG, _("No configuration files found, trying to use the default config source `%s'"), (char *)addresses->data);
     }
   
   if (addresses == NULL)
@@ -303,18 +297,16 @@ gconf_server_load_sources(void)
       /* We want to stay alive but do nothing, because otherwise every
          request would result in another failed gconfd being spawned.  
       */
-      const gchar* empty_addr[] = { NULL };
       gconf_log(GCL_ERR, _("No configuration sources in the source path, configuration won't be saved; edit "GCONF_CONFDIR"/path"));
       /* don't request error since there aren't any addresses */
-      sources = gconf_sources_new_from_addresses(empty_addr, NULL);
+      sources = gconf_sources_new_from_addresses(NULL, NULL);
 
       /* Install the sources as the default database */
       set_default_database (gconf_database_new(sources));
     }
   else
     {
-      sources = gconf_sources_new_from_addresses((const gchar**)addresses,
-                                                 &error);
+      sources = gconf_sources_new_from_addresses(addresses, &error);
 
       if (error != NULL)
         {
@@ -325,7 +317,7 @@ gconf_server_load_sources(void)
           error = NULL;
         }
       
-      g_free(addresses);
+      g_slist_free(addresses);
 
       g_assert(sources != NULL);
 
@@ -436,7 +428,7 @@ log_handler (const gchar   *log_domain,
 {
   GConfLogPriority pri = GCL_WARNING;
   
-  switch (log_level)
+  switch (log_level & G_LOG_LEVEL_MASK)
     {
     case G_LOG_LEVEL_ERROR:
     case G_LOG_LEVEL_CRITICAL:
@@ -468,29 +460,49 @@ main(int argc, char** argv)
 {
   struct sigaction act;
   sigset_t empty_mask;
-  /*   PortableServer_ObjectId objid = {0, sizeof("ConfigServer"), "ConfigServer"}; */
-  PortableServer_ObjectId* objid;
   CORBA_Environment ev;
   CORBA_ORB orb;
   gchar* logname;
   const gchar* username;
   guint len;
   gchar* ior;
-  OAF_RegistrationResult result;
   int exit_code = 0;
   GError *err;
   char *lock_dir;
   char *gconfd_dir;
-  ConfigServer other_server;
+  int dev_null_fd;
+  int write_byte_fd;
+  
+  /* Now this is an argument parser */
+  if (argc > 1)
+    write_byte_fd = atoi (argv[1]);
+  else
+    write_byte_fd = -1;
   
-  chdir ("/");
-
   /* This is so we don't prevent unmounting of devices. We divert
    * all messages to syslog
    */
-  close (0);
-  close (1);
-  close (2);
+  if (chdir ("/") < 0)
+    {
+       fprintf (stderr, "Could not change to root directory: %s\n",
+		g_strerror (errno));
+       exit (1);
+    }
+
+  if (!g_getenv ("GCONF_DEBUG_OUTPUT"))
+    {
+      dev_null_fd = open ("/dev/null", O_RDWR);
+      if (dev_null_fd >= 0)
+        {
+	  dup2 (dev_null_fd, 0);
+	  dup2 (dev_null_fd, 1);
+	  dup2 (dev_null_fd, 2);
+	}
+    }
+  else
+    {
+      gconf_log_debug_messages = TRUE;
+    }
   
   umask (022);
   
@@ -506,6 +518,12 @@ main(int argc, char** argv)
 
   g_log_set_handler (NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
                      log_handler, NULL);
+
+  g_log_set_handler ("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
+                     log_handler, NULL);
+
+  g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
+                     log_handler, NULL);
   
   /* openlog() does not copy logname - what total brokenness.
      So we free it at the end of main() */
@@ -535,23 +553,15 @@ main(int argc, char** argv)
 
   CORBA_exception_init(&ev);
 
-  if (!oaf_init(argc, argv))
-    {
-      gconf_log(GCL_ERR, _("Failed to init Object Activation Framework: please mail bug report to OAF maintainers"));
-      exit(1);
-    }
-
   init_databases ();
 
-  orb = oaf_orb_get();
+  orb = gconf_orb_get ();
   
-  POA_ConfigServer__init(&poa_server_servant, &ev);
+  POA_ConfigServer__init (&poa_server_servant, &ev);
   
   the_poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
   PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(the_poa, &ev), &ev);
 
-  objid = PortableServer_POA_activate_object(the_poa, &poa_server_servant, &ev);
-  
   server = PortableServer_POA_servant_to_reference(the_poa,
                                                    &poa_server_servant,
                                                    &ev);
@@ -566,92 +576,54 @@ main(int argc, char** argv)
   gconf_set_daemon_ior (ior);
   CORBA_free (ior);
 
-  gconfd_dir = g_strconcat (g_get_home_dir (), "/.gconfd", NULL);
-  lock_dir = g_strconcat (gconfd_dir, "/lock", NULL);
-
+  gconfd_dir = gconf_get_daemon_dir ();
+  lock_dir = gconf_get_lock_dir ();
+  
   if (mkdir (gconfd_dir, 0700) < 0 && errno != EEXIST)
     gconf_log (GCL_WARNING, _("Failed to create %s: %s"),
                gconfd_dir, g_strerror (errno));
   
   err = NULL;
-  daemon_lock = gconf_get_lock_or_current_holder (lock_dir,
-                                                  &other_server,
-                                                  &err);
 
+  daemon_lock = gconf_get_lock (lock_dir, &err);
+
+  if (daemon_lock != NULL)
+    {
+      /* This loads backends and so on. It needs to be done before
+       * we can handle any requests, so before we hit the
+       * main loop. if daemon_lock == NULL we won't hit the
+       * main loop.
+       */
+      gconf_server_load_sources ();
+    }
+  
+  /* notify caller that we're done either getting the lock
+   * or not getting it
+   */
+  if (write_byte_fd >= 0)
+    {
+      char buf[1] = { 'g' };
+      if (write (write_byte_fd, buf, 1) != 1)
+        {
+          gconf_log (GCL_ERR, _("Failed to write byte to pipe fd %d so client program may hang: %s"), write_byte_fd, g_strerror (errno));
+        }
+
+      close (write_byte_fd);
+    }
+  
   if (daemon_lock == NULL)
     {
       g_assert (err);
-      
-      /* Bad hack alert - register the current lockholder with oafd,
-       * since oafd seems to have forgotten about us.
-       */
-      gconf_log (GCL_WARNING, _("Failed to get lock for daemon: %s"),
+
+      gconf_log (GCL_WARNING, _("Failed to get lock for daemon, exiting: %s"),
                  err->message);
       g_error_free (err);
 
-      if (other_server != CORBA_OBJECT_NIL)
-        {
-          gconf_log (GCL_WARNING, _("Registering existing server with oafd, since OAF appears to have leaked it"));
-          
-          result = oaf_active_server_register (IID, other_server);
-          
-          if (result != OAF_REG_SUCCESS)
-            {
-              switch (result)
-                {
-                case OAF_REG_NOT_LISTED:
-                  gconf_log(GCL_ERR, _("OAF doesn't know about our IID; indicates broken installation; can't register existing server."));
-                  break;
-                  
-                case OAF_REG_ALREADY_ACTIVE:
-                  gconf_log(GCL_ERR, _("Another gconfd already registered with OAF, so we can't register the existing server"));
-                  break;
-                  
-                case OAF_REG_ERROR:
-                default:
-                  gconf_log(GCL_ERR, _("Unknown error registering existing gconfd with OAF; exiting"));
-                  break;
-                }
-            }
-
-          {
-            CORBA_Environment localev;
-            CORBA_exception_init (&localev);
-            CORBA_Object_release (server, &localev);
-            CORBA_exception_free (&localev);
-          }
-        }
-
-      return 1;
-    }
-  
-  /* Needs to be done right before registration,
-     after setting up the POA etc. */
-  gconf_server_load_sources();
-  
-  result = oaf_active_server_register(IID, server);
-
-  if (result != OAF_REG_SUCCESS)
-    {
-      switch (result)
-        {
-        case OAF_REG_NOT_LISTED:
-          gconf_log(GCL_ERR, _("OAF doesn't know about our IID; indicates broken installation; can't register; exiting\n"));
-          break;
-          
-        case OAF_REG_ALREADY_ACTIVE:
-          gconf_log(GCL_ERR, _("Another gconfd already registered with OAF; exiting\n"));
-          break;
-
-        case OAF_REG_ERROR:
-        default:
-          gconf_log(GCL_ERR, _("Unknown error registering gconfd with OAF; exiting\n"));
-          break;
-        }
       enter_shutdown ();
       shutdown_databases ();
+      
       return 1;
-    }
+    }  
 
   /* Read saved log file, if any */
   logfile_read ();
@@ -675,21 +647,13 @@ main(int argc, char** argv)
 
   gconfd_locale_cache_drop ();
 
-  /* Now we can unregister with OAF, after everything is fixed up. */  
-
-  if (server != CORBA_OBJECT_NIL)
-    {
-      oaf_active_server_unregister ("", server);
-      server = CORBA_OBJECT_NIL;
-    }
-
   if (daemon_lock)
     {
       err = NULL;
       gconf_release_lock (daemon_lock, &err);
       if (err != NULL)
         {
-          gconf_log (GCL_WARNING, _("Failed to release lockfile: %s"),
+          gconf_log (GCL_WARNING, _("Error releasing lockfile: %s"),
                      err->message);
           g_error_free (err);
         }
@@ -698,8 +662,9 @@ main(int argc, char** argv)
   daemon_lock = NULL;
   
   gconf_log (GCL_INFO, _("Exiting"));
-  
-  g_free (logname);
+
+  /* Can't do this due to stupid atexit() handler that calls g_log stuff */
+  /*   g_free (logname); */
   
   return exit_code;
 }
@@ -762,7 +727,7 @@ gconf_main(void)
 #ifdef GCONF_ENABLE_DEBUG
       gulong timeout_len = 1000*60*1; /* 1 sec * 60 s/min * 1 min */
 #else
-      gulong timeout_len = 1000*60*15; /* 1 sec * 60 s/min * 15 min */
+      gulong timeout_len = 1000*60*15; /* 1 sec * 60 s/min * 2 min */
 #endif
       
       g_assert(timeout_id == 0);
@@ -879,11 +844,11 @@ obtain_database (const gchar *address,
 {
   
   GConfSources* sources;
-  const gchar* addresses[] = { NULL, NULL };
+  GSList* addresses = NULL;
   GError* error = NULL;
   GConfDatabase *db;
 
-  addresses[0] = address;
+  g_slist_append(addresses, g_strdup(address));
   db = lookup_database (address);
 
   if (db)
@@ -1071,11 +1036,21 @@ gconf_set_exception(GError** error,
       case GCONF_ERROR_IN_SHUTDOWN:
         ce->err_no = ConfigInShutdown;
         break;
-        
+      case GCONF_ERROR_OVERRIDDEN:
+        ce->err_no = ConfigOverridden;
+        break;
+      case GCONF_ERROR_LOCK_FAILED:
+        ce->err_no = ConfigLockFailed;
+        break;
+
+      case GCONF_ERROR_OAF_ERROR:
+      case GCONF_ERROR_LOCAL_ENGINE:
       case GCONF_ERROR_NO_SERVER:
       case GCONF_ERROR_SUCCESS:
       default:
+        gconf_log (GCL_ERR, "Unhandled error code %d", en);
         g_assert_not_reached();
+        break;
       }
 
     CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
@@ -1665,9 +1640,7 @@ restore_client (const gchar *ior)
   
   CORBA_exception_init (&ev);
   
-  cl = CORBA_ORB_string_to_object (oaf_orb_get (),
-                                   (gchar*)ior,
-                                   &ev);
+  cl = CORBA_ORB_string_to_object (gconf_orb_get (), (gchar*)ior, &ev);
 
   CORBA_exception_free (&ev);
   
@@ -1714,9 +1687,7 @@ restore_listener (GConfDatabase* db,
   
   CORBA_exception_init (&ev);
   
-  cl = CORBA_ORB_string_to_object (oaf_orb_get (),
-                                   lle->ior,
-                                   &ev);
+  cl = CORBA_ORB_string_to_object (gconf_orb_get (), lle->ior, &ev);
 
   CORBA_exception_free (&ev);
   
@@ -1747,11 +1718,11 @@ restore_listener (GConfDatabase* db,
                  "Failed to cancel previous daemon's listener in saved state file: %s",
                  err->message);
       g_error_free (err);
-    }  
+    }
+  
+  new_cnxn = gconf_database_readd_listener (db, cl, "from-saved-state", lle->location);
 
-  new_cnxn = gconf_database_readd_listener (db, cl, lle->location);
-
-  gconf_log (GCL_DEBUG, "Attempting to update listener from saved state file, old connection %u, new connectin %u", (guint) lle->connection_id, (guint) new_cnxn);
+  gconf_log (GCL_DEBUG, "Attempting to update listener from saved state file, old connection %u, new connection %u", (guint) lle->connection_id, (guint) new_cnxn);
   
   ConfigListener_update_listener (cl,
                                   db->objref,
@@ -2130,8 +2101,8 @@ add_client (const ConfigListener client)
   gconfd_need_log_cleanup ();
   
   if (client_table == NULL)
-    client_table = g_hash_table_new ((GHashFunc) g_CORBA_Object_hash,
-                                     (GCompareFunc) g_CORBA_Object_equal);
+    client_table = g_hash_table_new ((GHashFunc) gconf_CORBA_Object_hash,
+                                     (GCompareFunc) gconf_CORBA_Object_equal);
 
   if (g_hash_table_lookup (client_table, client))
     {
@@ -2271,22 +2242,31 @@ drop_old_clients (void)
       while (tmp != NULL)
         {
           ConfigListener cl = tmp->data;
-          
-          ConfigListener_ping (cl, &ev);
+          CORBA_boolean result;
 
+          result = CORBA_Object_non_existent (cl, &ev);
+          
           if (ev._major != CORBA_NO_EXCEPTION)
             {
-              gconf_log (GCL_DEBUG, "Removing stale client");
+              gconf_log (GCL_WARNING, "Exception from CORBA_Object_non_existent(), assuming stale listener");
+              CORBA_exception_free (&ev);
+              CORBA_exception_init (&ev);
+              result = TRUE;
+            }
+
+          if (result)
+            {
+              gconf_log (GCL_DEBUG, "removing stale client in drop_old_clients");
               
               remove_client (cl);
-              
-              CORBA_exception_free (&ev);
             }
           
           tmp = g_slist_next (tmp);
         }
 
       g_slist_free (clients);
+
+      CORBA_exception_free (&ev);
     }
 }
 
